@@ -1,4 +1,4 @@
-// Copyright 2016-2017 Francesco Biscani (bluescarni@gmail.com)
+// Copyright 2016-2018 Francesco Biscani (bluescarni@gmail.com)
 //
 // This file is part of the mp++ library.
 //
@@ -25,8 +25,7 @@ inline namespace detail
 // http://en.cppreference.com/w/cpp/types/void_t
 #if MPPP_CPLUSPLUS >= 201703L
 
-template <typename... Ts>
-using void_t = std::void_t<Ts...>;
+using std::void_t;
 
 #else
 
@@ -72,14 +71,9 @@ using detected_t = typename detector<nonesuch, void, Op, Args...>::type;
 // http://en.cppreference.com/w/cpp/types/negation
 #if MPPP_CPLUSPLUS >= 201703L
 
-template <typename... Ts>
-using conjunction = std::conjunction<Ts...>;
-
-template <typename... Ts>
-using disjunction = std::disjunction<Ts...>;
-
-template <typename T>
-using negation = std::negation<T>;
+using std::conjunction;
+using std::disjunction;
+using std::negation;
 
 #else
 
@@ -116,14 +110,9 @@ struct negation : std::integral_constant<bool, !B::value> {
 // Small helpers, like C++14.
 #if MPPP_CPLUSPLUS >= 201402L
 
-template <bool B, typename T = void>
-using enable_if_t = std::enable_if_t<B, T>;
-
-template <typename T>
-using remove_cv_t = std::remove_cv_t<T>;
-
-template <typename T>
-using remove_extent_t = std::remove_extent_t<T>;
+using std::enable_if_t;
+using std::remove_cv_t;
+using std::remove_extent_t;
 
 #else
 
@@ -153,36 +142,31 @@ using is_ncrvr = conjunction<std::is_rvalue_reference<T>, negation<std::is_const
 // we will augment them with non-standard types defined
 // on some compilers.
 template <typename T>
-struct is_integral
-    : std::integral_constant<
-          bool, disjunction<std::is_integral<T>
+using is_integral = disjunction<std::is_integral<T>
 #if defined(MPPP_HAVE_GCC_INT128)
-                            ,
-                            // NOTE: for many of these type traits, the result must hold regardless
-                            // of the cv qualifications of T. Hence, we remove them.
-                            // http://eel.is/c++draft/meta.unary.cat
-                            std::is_same<remove_cv_t<T>, __int128_t>, std::is_same<remove_cv_t<T>, __uint128_t>
+                                ,
+                                // NOTE: for many of these type traits, the result must hold regardless
+                                // of the cv qualifications of T. Hence, we remove them.
+                                // http://eel.is/c++draft/meta.unary.cat
+                                std::is_same<remove_cv_t<T>, __int128_t>, std::is_same<remove_cv_t<T>, __uint128_t>
 #endif
-                            >::value> {
-};
+                                >;
 
 template <typename T>
-struct is_signed : std::integral_constant<bool, disjunction<std::is_signed<T>
+using is_signed = disjunction<std::is_signed<T>
 #if defined(MPPP_HAVE_GCC_INT128)
-                                                            ,
-                                                            std::is_same<remove_cv_t<T>, __int128_t>
+                              ,
+                              std::is_same<remove_cv_t<T>, __int128_t>
 #endif
-                                                            >::value> {
-};
+                              >;
 
 template <typename T>
-struct is_unsigned : std::integral_constant<bool, disjunction<std::is_unsigned<T>
+using is_unsigned = disjunction<std::is_unsigned<T>
 #if defined(MPPP_HAVE_GCC_INT128)
-                                                              ,
-                                                              std::is_same<remove_cv_t<T>, __uint128_t>
+                                ,
+                                std::is_same<remove_cv_t<T>, __uint128_t>
 #endif
-                                                              >::value> {
-};
+                                >;
 
 // make_unsigned machinery,
 template <typename T, typename = void>
@@ -190,6 +174,27 @@ struct make_unsigned_impl {
     using type = typename std::make_unsigned<T>::type;
 };
 
+// NOTE: before GCC 4.9.1 the specialisation of std::make_unsigned for wchar_t
+// is broken. See the bugreport here:
+// https://gcc.gnu.org/bugzilla/show_bug.cgi?format=multiple&id=60326
+//
+// It seems bits are also missing for char16_t and char32_t, but apparently
+// not in relation to make_unsigned (only make_signed, which we don't use
+// so far). This workaround is lifted directly from the commit cited
+// in the bugreport.
+#if __GNUC__ == 4 && (__GNUC_MINOR__ < 9 || (__GNUC_MINOR__ == 9 && __GNUC_PATCHLEVEL__ < 1))
+
+#if defined(_GLIBCXX_USE_WCHAR_T) && !defined(__WCHAR_UNSIGNED__)
+
+template <>
+struct make_unsigned_impl<wchar_t> {
+    using type = typename make_unsigned_impl<__WCHAR_TYPE__>::type;
+};
+
+#endif
+
+#endif
+
 #if defined(MPPP_HAVE_GCC_INT128)
 
 // NOTE: make_unsigned is supposed to preserve cv qualifiers, hence the non-trivial implementation.
