@@ -1,4 +1,4 @@
-// Copyright 2016-2017 Francesco Biscani (bluescarni@gmail.com)
+// Copyright 2016-2018 Francesco Biscani (bluescarni@gmail.com)
 //
 // This file is part of the mp++ library.
 //
@@ -37,6 +37,7 @@
 
 #include <cassert>
 #include <cstddef>
+#include <exception>
 #include <iostream>
 #include <limits>
 #include <memory>
@@ -110,7 +111,8 @@ inline void cleanup()
 
 /// Initialisation function for the pybind11 integration.
 /**
- * This function should be called right after the ``PYBIND11_MODULE()`` invocation.
+ * This function should be called in the definition of a pybind11 extension module, right after the
+ * ``PYBIND11_MODULE()`` invocation.
  */
 inline void init()
 {
@@ -176,6 +178,17 @@ inline void init()
     auto fraction_class = py::module::import("fractions").attr("Fraction");
     globals::fraction_class.reset(new py::object(std::move(fraction_class)));
 
+    // Tanslation for mp++ exceptions.
+    py::register_exception_translator([](std::exception_ptr p) {
+        try {
+            if (p) {
+                std::rethrow_exception(p);
+            }
+        } catch (const mppp::zero_division_error &e) {
+            ::PyErr_SetString(::PyExc_ZeroDivisionError, e.what());
+        }
+    });
+
     // Mark as inited.
     globals::inited = true;
 
@@ -427,8 +440,8 @@ struct type_caster<mppp::real128> {
             || !::PyObject_IsInstance(src.ptr(), mppp_pybind11::globals::mpf_class->ptr())) {
             return false;
         }
-        const auto prec = src.attr("context").attr("prec").cast<::mpfr_prec_t>();
-        if (prec != mppp::safe_cast<::mpfr_prec_t>(mppp::real128_sig_digits())) {
+        const auto prec = src.attr("context").attr("prec").cast<decltype(mppp::real128_sig_digits())>();
+        if (prec != mppp::real128_sig_digits()) {
             return false;
         }
         const auto mpf_tuple = src.attr("_mpf_").cast<tuple>();
@@ -470,8 +483,8 @@ struct type_caster<mppp::real128> {
         if (!mppp_pybind11::globals::mpmath) {
             throw std::runtime_error("Cannot convert a real128 to an mpf if mpmath is not available");
         }
-        const auto prec = mppp_pybind11::globals::mpmath_mp->attr("prec").cast<::mpfr_prec_t>();
-        if (prec != mppp::safe_cast<::mpfr_prec_t>(mppp::real128_sig_digits())) {
+        const auto prec = mppp_pybind11::globals::mpmath_mp->attr("prec").cast<decltype(mppp::real128_sig_digits())>();
+        if (prec != mppp::real128_sig_digits()) {
             throw std::invalid_argument(
                 "Cannot convert the real128 " + src.to_string() + " to an mpf: the precision of real128 ("
                 + std::to_string(mppp::real128_sig_digits()) + ") is different from the current mpf precision ("
